%% program solves a portfolio problem for an agent with
%% first-order risk aversion preferences (R-FORA)
%% creates the '+' signs in Figure 2

%% gross risk-free rate
rf = 1.02;
%% mean and standard deviation of the log stock market return
rm = 0.06;
rs = 0.2;
%% mean and standard deviation of the log return on the
%% non-financial asset
nm = 0.04;
ns = 0.03;
%% correlation between the log returns on the stock market and
%% the non-financial asset
omg = 0.1;
%% fixed fraction of wealth allocated to the non-financial asset
thn = 0.75;

%% parameters of the aggregator function W(.,.)
%% beta (bet) and rho (rho)
bet = 0.9;
rho = -1;

%% Figure 2 looks at a range of values of the
%% preference parameters gamma (gm) and lambda (lam)
gm_min = 1.1;
gm_max = 150;
lam_min = 1.1;
lam_max = 150;

%% fineness of grid discretizing the (gamma,lambda) plane
lim = 40;

%% fineness of grid used to discretize the area being integrated over
grd_fin = 301;
stp = 15/(grd_fin-1);
grd = [-7.5:stp:7.5]';
grd1 = kron(grd,ones(grd_fin,1));
grd2 = kron(ones(grd_fin,1),grd);

%% the weighting vector for Simpson's rule (double integration)
s1 = 4*kron(ones((grd_fin-3)/2,1),[1;0])+2*kron(ones((grd_fin-3)/2,1),[0;1]);
s1 = [1; s1; 4; 1];
s2 = kron(s1,ones(grd_fin,1));
s3 = kron(ones(grd_fin,1),s1);
s4 = s2.*s3;

%% array marking the range of (gamma,lambda) plane where the agent
%% does not participate in the stock market
dgn = zeros(lim,lim);

for count = 1:lim;
   count
   for count2 = 1:lim;
      gm = gm_min + (count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min + (count2-1)*(lam_max-lam_min)/(lim-1);

t1 = [];

%% "ths" is the agent's optimal allocation to the stock market
%% try ths = -0.1% and 0.1%
%% if utility at the former is higher than at the latter, this tells us
%% (appoximately) that non-participation is optimal
for ths = -0.001:0.002:0.001;
   %% for given ths, compute the certainty equivalent mu in equation (52) 
   %% certainty equivalent is defined in equation (4)
   u1 = (1-ths-thn)*rf+thn*exp(nm+ns*grd1)+ths*exp(rm+rs*grd2);
   u2 = (u1.^(1-gm)).*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
   u3 = (u2'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
   %% starting values for search
   m_0 = 0.9;
   m_1 = 1.15;
   %% now compute the function in (4) at each endpoint
   %% first endpoint
   w1 = u1.^(1-gm) - m_0^(1-gm);
   w2 = (1-sign(u1-m_0))/2;
   w3 = w1.*w2.*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
   w4 = (w3'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
   f_0 = u3 + (lam-1)*w4 - m_0^(1-gm);
   %% second endpoint
   w1 = u1.^(1-gm) - m_1^(1-gm);
   w2 = (1-sign(u1-m_1))/2;
   w3 = w1.*w2.*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
   w4 = (w3'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
   f_1 = u3 + (lam-1)*w4 - m_1^(1-gm);
   %% now use interval bisection to find the certainty equivalent
   flg = 1;
   ct = 1;
   while flg > 0.0000001;
      ct = ct + 1;
      m_2 = (m_0+m_1)/2;
      flg = abs(m_2-m_1);
      w1 = u1.^(1-gm)-m_2^(1-gm);
	  w2 = (1-sign(u1-m_2))/2;
      w3 = w1.*w2.*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
      w4 = (w3'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
      f_2 = u3 + (lam-1)*w4 - m_2^(1-gm);
      if sign(f_1) == sign(f_2);
         m_1 = m_2;
         f_1 = f_2;
      else;
         m_0 = m_1;
         m_1 = m_2;
         f_0 = f_1;
         f_1 = f_2;
      end;
   end;
   t1 = [t1; m_1];
end;

%% locate the allocation which maximizes the certainty equivalent
[q1 q2] = max(t1);

%% records the non-participation area
if q2 == 1;
    dgn(count,count2) = 1;
end;

end;
end;

plt = [];

for count = 1:lim;
   for count2 = 1:lim;
      gm = gm_min+(count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min+(count2-1)*(lam_max-lam_min)/(lim-1);
      if dgn(count,count2) == 1;
         plt = [plt; gm lam];
      end;
   end;
end;

figure;

plot(plt(:,1),plt(:,2),'+');
axis([gm_min gm_max lam_min lam_max]);
xlabel('\gamma');
ylabel('\lambda');
